@@ -9,11 +9,11 @@ Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.18
+Version 1.24
 
 =cut
 
-our $VERSION = '1.20';
+our $VERSION = '1.24';
 
 =head1 SYNOPSIS
 
@@ -77,23 +77,71 @@ my $Test = Test::Builder->new();
 
 =head1 CONSTRUCTOR
 
-=head2 new
+=head2 new( %args )
 
 Behaves like, and calls, L<WWW::Mechanize>'s C<new> method.  Any parms
 passed in get passed to WWW::Mechanize's constructor.
 
+You can pass in C<< autolint => 1 >> to make Test::WWW::Mechanize
+automatically run HTML::Lint after any of the following methods are
+called.
+
+=over
+
+=item * get_ok()
+
+=back
+
+and will eventually do the same after any of the following:
+
+=over
+
+=item * post_ok()
+
+=item * back_ok()
+
+=item * submit_form_ok()
+
+=item * follow_link_ok()
+
+=item * click_ok()
+
+=back
+
+This means you no longerhave to do the following:
+
+    my $mech = Test::WWW::Mechanize->new();
+    $mech->get_ok( $url, 'Fetch the intro page' );
+    $mech->html_lint_ok( 'Intro page looks OK' );
+
+and can simply do
+
+    my $mech = Test::WWW::Mechanize->new( autolint => 1 );
+    $mech->get_ok( $url, 'Fetch the intro page' );
+
+The C<< $mech->get_ok() >> only counts as one test in the test count.  Both the
+main IO operation and the linting must pass for the entire test to pass.
+
 =cut
 
 sub new {
     my $class = shift;
-    my %mech_args = @_;
 
-    my $self = $class->SUPER::new( %mech_args );
+    my %args = (
+        agent => "Test-WWW-Mechanize/$VERSION",
+        @_
+    );
+
+    my $autolint = delete $args{autolint};
+
+    my $self = $class->SUPER::new( %args );
+
+    $self->{autolint} = $autolint;
 
     return $self;
 }
 
-=head1 METHODS: GETTING & POSTING
+=head1 METHODS: HTTP VERBS
 
 =head2 $mech->get_ok($url, [ \%LWP_options ,] $desc)
 
@@ -139,6 +187,80 @@ sub get_ok {
         $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
         $desc = "GET $url";
     }
+
+    $ok = $self->_maybe_lint( $ok, $desc );
+
+    return $ok;
+}
+
+sub _maybe_lint {
+    my $self = shift;
+    my $ok   = shift;
+    my $desc = shift;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    if ( $ok ) {
+        if ( $self->is_html && $self->{autolint} ) {
+            $ok = $self->_lint_content_ok( $desc );
+        }
+        else {
+            $Test->ok( $ok, $desc );
+        }
+    }
+    else {
+        $Test->ok( $ok, $desc );
+        $Test->diag( $self->status );
+        $Test->diag( $self->response->message ) if $self->response;
+    }
+
+    return $ok;
+}
+
+=head2 $mech->head_ok($url, [ \%LWP_options ,] $desc)
+
+A wrapper around WWW::Mechanize's head(), with similar options, except
+the second argument needs to be a hash reference, not a hash. Like
+well-behaved C<*_ok()> functions, it returns true if the test passed,
+or false if not.
+
+A default description of "HEAD $url" is used if none if provided.
+
+=cut
+
+sub head_ok {
+    my $self = shift;
+    my $url = shift;
+
+    my $desc;
+    my %opts;
+
+    if ( @_ ) {
+        my $flex = shift; # The flexible argument
+
+        if ( !defined( $flex ) ) {
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'HASH' ) {
+            %opts = %{$flex};
+            $desc = shift;
+        }
+       elsif ( ref $flex eq 'ARRAY' ) {
+            %opts = @{$flex};
+            $desc = shift;
+        }
+        else {
+            $desc = $flex;
+        }
+    } # parms left
+
+    $self->head( $url, %opts );
+    my $ok = $self->success;
+
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "HEAD $url";
+    }
     $Test->ok( $ok, $desc );
     if ( !$ok ) {
         $Test->diag( $self->status );
@@ -200,8 +322,59 @@ sub post_ok {
     return $ok;
 }
 
+=head2 $mech->put_ok( $url, [ \%LWP_options ,] $desc )
 
-=head2 submit_form_ok( \%parms [, $desc] )
+A wrapper around WWW::Mechanize's put(), with similar options, except
+the second argument needs to be a hash reference, not a hash. Like
+well-behaved C<*_ok()> functions, it returns true if the test passed,
+or false if not.
+
+A default description of "PUT to $url" is used if none if provided.
+
+=cut
+
+sub put_ok {
+    my $self = shift;
+    my $url = shift;
+
+    my $desc;
+    my %opts;
+
+    if ( @_ ) {
+        my $flex = shift; # The flexible argument
+
+        if ( !defined( $flex ) ) {
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'HASH' ) {
+            %opts = %{$flex};
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'ARRAY' ) {
+            %opts = @{$flex};
+            $desc = shift;
+        }
+        else {
+            $desc = $flex;
+        }
+    } # parms left
+
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "PUT $url";
+    }
+    $self->put( $url, \%opts );
+    my $ok = $self->success;
+    $Test->ok( $ok, $desc );
+    if ( !$ok ) {
+        $Test->diag( $self->status );
+        $Test->diag( $self->response->message ) if $self->response;
+    }
+
+    return $ok;
+}
+
+=head2 $mech->submit_form_ok( \%parms [, $desc] )
 
 Makes a C<submit_form()> call and executes tests on the results.
 The form must be found, and then submitted successfully.  Otherwise,
@@ -228,7 +401,7 @@ sub submit_form_ok {
     my $desc = shift;
 
     if ( ref $parms ne 'HASH' ) {
-       Carp::croak "FATAL: parameters must be given as a hashref";
+       Carp::croak 'FATAL: parameters must be given as a hashref';
     }
 
     # return from submit_form() is an HTTP::Response or undef
@@ -237,7 +410,7 @@ sub submit_form_ok {
     my $ok;
     my $error;
     if ( !$response ) {
-        $error = "No matching form found";
+        $error = 'No matching form found';
     }
     else {
         if ( $response->is_success ) {
@@ -282,12 +455,12 @@ sub follow_link_ok {
     my $desc = shift;
 
     if (!defined($desc)) {
-        my $parms_str = join(", ", map { join("=", $_, $parms->{$_}) } keys(%{$parms}));
-        $desc = "Followed link with '$parms_str'" if !defined($desc);
+        my $parms_str = join(', ', map { join('=', $_, $parms->{$_}) } keys(%{$parms}));
+        $desc = qq{Followed link with "$parms_str"} if !defined($desc);
     }
 
     if ( ref $parms ne 'HASH' ) {
-       Carp::croak "FATAL: parameters must be given as a hashref";
+       Carp::croak 'FATAL: parameters must be given as a hashref';
     }
 
     # return from follow_link() is an HTTP::Response or undef
@@ -296,7 +469,7 @@ sub follow_link_ok {
     my $ok;
     my $error;
     if ( !$response ) {
-        $error = "No matching link found";
+        $error = 'No matching link found';
     }
     else {
         if ( $response->is_success ) {
@@ -313,12 +486,38 @@ sub follow_link_ok {
     return $ok;
 }
 
+
+=head2 click_ok( $button[, $desc] )
+
+Clicks the button named by C<$button>.  An optional C<$desc> can
+be given for the test.
+
+=cut
+
+sub click_ok {
+    my $self   = shift;
+    my $button = shift;
+    my $desc   = shift;
+
+    my $response = $self->click( $button );
+    if ( !$response ) {
+        return $Test->ok( 0, $desc );
+    }
+
+    if ( !$response->is_success ) {
+        $Test->diag( "Failed test $desc:" );
+        $Test->diag( $response->as_string );
+        return $Test->ok( 0, $desc );
+    }
+    return $Test->ok( 1, $desc );
+}
+
 =head1 METHODS: CONTENT CHECKING
 
-=head2 $mech->html_lint_ok( [$msg] )
+=head2 $mech->html_lint_ok( [$desc] )
 
 Checks the validity of the HTML on the current page.  If the page is not
-HTML, then it fails.  The URI is automatically appended to the I<$msg>.
+HTML, then it fails.  The URI is automatically appended to the I<$desc>.
 
 Note that HTML::Lint must be installed for this to work.  Otherwise,
 it will blow up.
@@ -327,42 +526,54 @@ it will blow up.
 
 sub html_lint_ok {
     my $self = shift;
-    my $msg = shift;
-
-    eval 'require HTML::Lint';
-    $@ and die 'html_lint_ok cannot run without HTML::Lint';
+    my $desc = shift;
 
     my $uri = $self->uri;
-    $msg = $msg ? "$msg ($uri)" : $uri;
+    $desc = $desc ? "$desc ($uri)" : $uri;
 
     my $ok;
 
     if ( $self->is_html ) {
-
-        my $lint = HTML::Lint->new;
-        $lint->parse( $self->content );
-
-        my @errors = $lint->errors;
-        my $nerrors = @errors;
-        if ( $nerrors ) {
-            $ok = $Test->ok( 0, $msg );
-            $Test->diag( "HTML::Lint errors for $uri" );
-            $Test->diag( $_->as_string ) for @errors;
-            my $s = $nerrors == 1 ? '' : 's';
-            $Test->diag( "$nerrors error$s on the page" );
-        }
-        else {
-            $ok = $Test->ok( 1, $msg );
-        }
+        $ok = $self->_lint_content_ok( $desc );
     }
     else {
-        $ok = $Test->ok( 0, $msg );
+        $ok = $Test->ok( 0, $desc );
         $Test->diag( q{This page doesn't appear to be HTML, or didn't get the proper text/html content type returned.} );
     }
 
     return $ok;
 }
 
+sub _lint_content_ok {
+    my $self = shift;
+    my $desc = shift;
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    if ( not ( eval 'require HTML::Lint' ) ) {
+        die "Test::WWW::Mechanize can't do linting without HTML::Lint: $@";
+    }
+
+    # XXX Combine with the cut'n'paste version in get_ok()
+    my $lint = HTML::Lint->new;
+    $lint->parse( $self->content );
+
+    my @errors = $lint->errors;
+    my $nerrors = @errors;
+    my $ok;
+    if ( $nerrors ) {
+        $ok = $Test->ok( 0, $desc );
+        $Test->diag( 'HTML::Lint errors for ' . $self->uri );
+        $Test->diag( $_->as_string ) for @errors;
+        my $s = $nerrors == 1 ? '' : 's';
+        $Test->diag( "$nerrors error$s on the page" );
+    }
+    else {
+        $ok = $Test->ok( 1, $desc );
+    }
+
+    return $ok;
+}
 
 =head2 $mech->title_is( $str [, $desc ] )
 
@@ -376,7 +587,7 @@ sub title_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
-    $desc = "Title is '$str'" if !defined($desc);
+    $desc = qq{Title is "$str"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->title, $str, $desc );
@@ -394,7 +605,7 @@ sub title_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Title is like '$regex'" if !defined($desc);
+    $desc = qq{Title is like "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->title, $regex, $desc );
@@ -412,7 +623,7 @@ sub title_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Title unlike '$regex'" if !defined($desc);
+    $desc = qq{Title is unlike "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->title, $regex, $desc );
@@ -430,7 +641,7 @@ sub base_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
-    $desc = "Base is '$str'" if !defined($desc);
+    $desc = qq{Base is "$str"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->base, $str, $desc );
@@ -448,7 +659,7 @@ sub base_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Base is like '$regex'" if !defined($desc);
+    $desc = qq{Base is like "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->base, $regex, $desc );
@@ -466,7 +677,7 @@ sub base_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Base unlike '$regex'" if !defined($desc);
+    $desc = qq{Base is unlike "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->base, $regex, $desc );
@@ -502,7 +713,7 @@ sub content_contains {
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     if ( ref($str) eq 'REGEX' ) {
-        diag( "content_contains takes a string, not a regex" );
+        diag( 'content_contains takes a string, not a regex' );
     }
     $desc = qq{Content contains "$str"} if !defined($desc);
 
@@ -539,7 +750,7 @@ sub content_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Content is like '$regex'" if !defined($desc);
+    $desc = qq{Content is like "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->content, $regex, $desc );
@@ -555,7 +766,7 @@ sub content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Content unlike '$regex'" if !defined($desc);
+    $desc = qq{Content is like "$regex"} if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->content, $regex, $desc );
@@ -572,7 +783,7 @@ sub has_tag {
     my $tag  = shift;
     my $text = shift;
     my $desc = shift;
-    $desc = "Page has $tag tag with '$text'" if !defined($desc);
+    $desc = qq{Page has $tag tag with "$text"} if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $text eq $_[0] } );
 
@@ -591,7 +802,7 @@ sub has_tag_like {
     my $tag  = shift;
     my $regex = shift;
     my $desc = shift;
-    $desc = "Page has $tag tag like '$regex'" if !defined($desc);
+    $desc = qq{Page has $tag tag like "$regex"} if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $_[0] =~ $regex } );
 
@@ -623,7 +834,7 @@ https links.
 sub followable_links {
     my $self = shift;
 
-    return $self->find_all_links( url_abs_regex => qr[^https?://] );
+    return $self->find_all_links( url_abs_regex => qr{^https?://} );
 }
 
 =head2 $mech->page_links_ok( [ $desc ] )
@@ -893,8 +1104,8 @@ sub link_content_unlike {
 # Create a default description for the link_* methods, including the link count.
 sub _default_links_desc {
     my ($urls, $desc_suffix) = @_;
-    my $url_count = scalar(@$urls);
-    return sprintf("%d link%s %s", $url_count, $url_count == 1 ? "" : "s", $desc_suffix);
+    my $url_count = scalar(@{$urls});
+    return sprintf( '%d link%s %s', $url_count, $url_count == 1 ? '' : 's', $desc_suffix );
 }
 
 # This actually performs the status check of each url.
@@ -910,7 +1121,7 @@ sub _check_links_status {
 
     my @failures;
 
-    for my $url ( @$urls ) {
+    for my $url ( @{$urls} ) {
         if ( $mech->follow_link( url => $url ) ) {
             if ( $test eq 'is' ) {
                 push( @failures, $url ) unless $mech->status() == $status;
@@ -940,7 +1151,7 @@ sub _check_links_content {
     my $mech = $self->clone();
 
     my @failures;
-    for my $url ( @$urls ) {
+    for my $url ( @{$urls} ) {
         if ( $mech->follow_link( url => $url ) ) {
             my $content=$mech->content();
             if ( $test eq 'like' ) {
@@ -1181,7 +1392,7 @@ and Pete Krawczyk for patches.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2004-2007 Andy Lester, all rights reserved.
+Copyright 2004-2008 Andy Lester, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
